"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const chalk = require("chalk");
const dateFns = require("date-fns");
const pathHelper_1 = require("./../helpers/pathHelper");
const utils_1 = require("./../helpers/utils");
const envHelper_1 = require("../helpers/envHelper");
const config_1 = require("../config/config");
const packagesHelper_1 = require("../helpers/packagesHelper");
const install_1 = require("../scripts/install");
const build_1 = require("../scripts/build");
exports.default = {
    command: 'deploy',
    describe: 'Deploy project for production',
    handler: commandHandler,
    builder: commandBuilder
};
function commandBuilder(yargs) {
    return yargs
        .option('stop', {
        description: 'Stop running application process for local deployment'
    })
        .option('target', {
        alias: 't',
        description: 'Deployment target, supported targets are heroku/now/local, local is deafult'
    })
        .option('remote', {
        alias: 'r',
        description: 'For heroku only, specify remote name, heroku by default'
    })
        .option('skip-client-build', {
        alias: 'scb',
        description: 'Skip client build'
    })
        .example('deploy', 'Deploy project for production (starts project with one of supported process managers).')
        .example('deploy --stop', 'Stop running app (it is removed from process list and cannot be restarted again).')
        .example('deploy -t heroku -r dev', 'Deploys app to heroku server using dev remote.');
}
function commandHandler(argv) {
    return __awaiter(this, void 0, void 0, function* () {
        envHelper_1.default.checkFolderStructure();
        let processManager = detectProcessManager();
        let appName = envHelper_1.default.getAppName();
        let remote = argv.remote;
        if (argv.stop) {
            return utils_1.default.logOperation(`Stop '${appName}' application process`, () => {
                stopApp(processManager, appName);
            });
        }
        let target = 'local';
        if (argv.target) {
            //TODO check target is one of supported values
            target = argv.target;
            if (target === 'heroku') {
                if (!argv.remote) {
                    utils_1.default.logAndExit('Specify remote for heroku deployments');
                }
            }
        }
        let deployDir = pathHelper_1.default.projectRelative(config_1.default.paths.deploy.root);
        let buildDir = pathHelper_1.default.projectRelative(config_1.default.paths.build.root);
        let deployParams = {
            target,
            appName,
            deployDir,
            buildDir,
            processManager,
            remote
        };
        try {
            beforeDeploy(deployParams);
            yield ensureBuild(argv.skipClientBuild);
            if (fs.existsSync(deployDir)) {
                stopApp(processManager, appName);
            }
            utils_1.default.logOperation('Copy build assets', () => {
                let isFirstDeploy = !fs.existsSync(deployDir);
                if (isFirstDeploy) {
                    utils_1.default.ensureEmptyDir(deployDir);
                }
                else {
                    let localDir = pathHelper_1.default.deployRelative(config_1.default.paths.server.local);
                    let gitDir = `!${deployDir}/.git`;
                    utils_1.default.clearDir(deployDir, [localDir, gitDir]);
                }
                fs.copySync(buildDir, deployDir);
            });
            afterDeploy(deployParams);
        }
        catch (err) {
            utils_1.default.logAndExit(err);
        }
    });
}
function beforeDeploy(deployParams) {
    const { target, processManager, appName, remote, deployDir } = deployParams;
    switch (target) {
        case 'local':
            break;
        case 'heroku':
            utils_1.default.runCommand('git', ['checkout', remote], {
                title: `Switch to "${remote}" branch`,
                path: deployDir
            });
            break;
        default:
            throw new Error(`Unsupported target: "${target}"`);
            break;
    }
}
function afterDeploy(deployParams) {
    const { target, processManager, appName, remote, deployDir } = deployParams;
    switch (target) {
        case 'local':
            //install packages there
            let installCommandInfo = packagesHelper_1.default.getInstallPackagesCommand();
            utils_1.default.runCommand(installCommandInfo.command, installCommandInfo.params, {
                title: 'Install production dependencies',
                path: deployParams.deployDir
            });
            startApp(processManager, appName);
            break;
        case 'heroku':
            utils_1.default.runCommand('git', ['pull', remote], {
                title: `Pull changes from remote`,
                path: deployDir
            });
            utils_1.default.runCommand('git', ['add', '.'], {
                title: 'Add files to git',
                path: deployDir
            });
            utils_1.default.runCommand('git', ['commit', '-m', `"Deployment at ${dateFns.format(new Date(), 'YYYY-MM-DDTHH:mm:ss')}"`], {
                title: 'Commit files to git',
                ignoreError: true,
                path: deployDir
            });
            utils_1.default.runCommand('git', ['push', remote, `${remote}:master`], {
                title: 'Deploying to Heroku...',
                showOutput: true,
                path: deployDir
            });
            break;
        default:
            throw new Error(`Unsupported target: "${target}"`);
            break;
    }
}
function ensureBuild(skipClientBuild) {
    //return Promise.resolve(null);
    install_1.default.installAll();
    return build_1.default.build(skipClientBuild ? 'server' : 'full');
}
function detectProcessManager() {
    let processManager = utils_1.default.findGlobalCommandByPrecedence(['pm2', 'forever']);
    if (!processManager) {
        utils_1.default.logAndExit(`Install globally one of supported process managers: forever or pm2.`);
    }
    return processManager;
}
function stopApp(processManager, appName) {
    let params = [];
    if (processManager === 'forever') {
        params = ['stop', appName];
    }
    if (processManager === 'pm2') {
        params = ['delete', appName];
    }
    utils_1.default.runCommand(processManager, params, {
        path: pathHelper_1.default.projectRelative(config_1.default.paths.deploy.root),
        ignoreError: true,
        showOutput: false
    });
}
function startApp(processManager, appName) {
    let params = [];
    if (processManager === 'forever') {
        params = ['start', '--id', appName, 'index.js'];
    }
    if (processManager === 'pm2') {
        params = ['start', 'index.js', '--name', appName];
    }
    utils_1.default.runCommand(processManager, params, {
        title: 'Start process',
        path: pathHelper_1.default.projectRelative(config_1.default.paths.deploy.root)
    });
    utils_1.default.logAndExit(`Process has been started. By default it is available on ${chalk.cyan('localhost:5000')}.`);
}
//# sourceMappingURL=deploy.js.map